@@ -1,6 +1,6 @@
 /* channel.c
  *
- * $Id: channel.c,v 1.94 2001/03/12 21:58:47 nisse Exp $
+ * $Id: channel.c,v 1.98 2001/04/11 05:38:53 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -65,22 +65,6 @@ make_channel_open_exception(UINT32 error_code, const char *msg)
   return &self->super;
 }
 
-/* GABA:
-   (class
-     (name connection_service)
-     (super command)
-     (vars
-       ; Supported global requests 
-       (global_requests object alist)
-
-       (channel_types object alist) ))
-
-       ; Initialize connection (for instance, request channels to be 
-       ; opened or services to be forwarded).
-
-       ; (start object connection_startup)))
-*/
-
 
 struct lsh_string *
 format_global_failure(void)
@@ -192,10 +176,6 @@ do_exc_finish_channel_handler(struct exception_handler *s,
        * code in a live channel is supposed to raise
        * EXC_FINISH_PENDING. The typical caller is a channel's
        * CHANNEL_CLOSE callback that is called below. */
-#if 0
-      if (!self->connection->table->channel_count)
-	EXCEPTION_RAISE(self->connection->e, &finish_read_exception);
-#endif
       break;
       
     case EXC_FINISH_CHANNEL:
@@ -463,16 +443,6 @@ channel_start_receive(struct ssh_channel *channel,
 	    (channel, initial_window_size - channel->rec_window_size));
 }
 
-
-/* Ugly macros to make it a little simpler to free the input packet at
- * the right time. */
-
-#define RETURN goto foo_finish
-#define END(s) do { foo_finish: \
-                    lsh_string_free((s)); \
-                    return; } while(0)
-
-
 /* Channel related messages */
 
 /* GABA:
@@ -624,8 +594,6 @@ DEFINE_PACKET_HANDLER(static, global_request_handler, connection, packet)
 	}
       if (!req)
 	{
-	  lsh_string_free(packet);
-
 	  C_WRITE(connection, format_global_failure());
 	  return;
 	}
@@ -655,10 +623,7 @@ DEFINE_PACKET_HANDLER(static, global_request_handler, connection, packet)
 	}
     }
   else
-    {
-      PROTOCOL_ERROR(connection->e, "Invalid SSH_MSG_GLOBAL_REQUEST message.");
-    }
-  lsh_string_free(packet);
+    PROTOCOL_ERROR(connection->e, "Invalid SSH_MSG_GLOBAL_REQUEST message.");
 }
 
 DEFINE_PACKET_HANDLER(static, global_success_handler,
@@ -667,7 +632,7 @@ DEFINE_PACKET_HANDLER(static, global_success_handler,
   if (packet->length != 1)
     {
       PROTOCOL_ERROR(connection->e, "Invalid GLOBAL_REQUEST_SUCCESS message.");
-      RETURN;
+      return;
     }
 
   assert(packet->data[0] == SSH_MSG_REQUEST_SUCCESS);
@@ -682,7 +647,6 @@ DEFINE_PACKET_HANDLER(static, global_success_handler,
 		 object_queue_remove_head(&connection->table->pending_global_requests));
     COMMAND_RETURN(ctx->c, connection);
   }
-  END(packet);
 }
 
 struct exception global_request_exception =
@@ -694,7 +658,7 @@ DEFINE_PACKET_HANDLER(static, global_failure_handler,
   if (packet->length != 1)
     {
       PROTOCOL_ERROR(connection->e, "Invalid GLOBAL_REQUEST_FAILURE message.");
-      RETURN;
+      return;
     }
 
   assert(packet->data[0] == SSH_MSG_REQUEST_FAILURE);
@@ -709,7 +673,6 @@ DEFINE_PACKET_HANDLER(static, global_failure_handler,
 		   object_queue_remove_head(&connection->table->pending_global_requests));
       EXCEPTION_RAISE(ctx->e, &global_request_exception);
     }
-  END(packet);
 }
 
 /* FIXME: Don't store the channel here, instead have it passed as the
@@ -920,8 +883,6 @@ DEFINE_PACKET_HANDLER(static, channel_request_handler,
     }
   else
     PROTOCOL_ERROR(connection->e, "Invalid SSH_MSG_CHANNEL_REQUEST message.");
-  
-  lsh_string_free(packet);
 }
 
 
@@ -1151,8 +1112,6 @@ DEFINE_PACKET_HANDLER(static, channel_open_handler,
     }
   else
     PROTOCOL_ERROR(connection->e, "Invalid SSH_MSG_CHANNEL_OPEN message.");
-
-  lsh_string_free(packet);
 }     
 
 DEFINE_PACKET_HANDLER(static, window_adjust_handler,
@@ -1174,8 +1133,6 @@ DEFINE_PACKET_HANDLER(static, window_adjust_handler,
       struct ssh_channel *channel = lookup_channel(connection->table,
 						   channel_number);
 
-      lsh_string_free(packet);
-      
       if (channel
 	  && !(channel->flags & CHANNEL_RECEIVED_CLOSE))
 	{
@@ -1197,10 +1154,7 @@ DEFINE_PACKET_HANDLER(static, window_adjust_handler,
 	}
     }
   else
-    {
-      lsh_string_free(packet);
-      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_WINDOW_ADJUST message.");
-    }
+    PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_WINDOW_ADJUST message.");
 }
 
 DEFINE_PACKET_HANDLER(static, channel_data_handler,
@@ -1222,8 +1176,6 @@ DEFINE_PACKET_HANDLER(static, channel_data_handler,
       struct ssh_channel *channel = lookup_channel(connection->table,
 						   channel_number);
 
-      lsh_string_free(packet);
-      
       if (channel && channel->receive
 	  && !(channel->flags & (CHANNEL_RECEIVED_EOF
 				 | CHANNEL_RECEIVED_CLOSE)))
@@ -1271,10 +1223,7 @@ DEFINE_PACKET_HANDLER(static, channel_data_handler,
 	}
     }
   else
-    {
-      lsh_string_free(packet);
-      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_DATA message.");
-    }
+    PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_DATA message.");
 }
 
 DEFINE_PACKET_HANDLER(static, channel_extended_data_handler,
@@ -1298,8 +1247,6 @@ DEFINE_PACKET_HANDLER(static, channel_extended_data_handler,
       struct ssh_channel *channel = lookup_channel(connection->table,
 						   channel_number);
 
-      lsh_string_free(packet);
-      
       if (channel && channel->receive
 	  && !(channel->flags & (CHANNEL_RECEIVED_EOF
 				 | CHANNEL_RECEIVED_CLOSE)))
@@ -1357,10 +1304,7 @@ DEFINE_PACKET_HANDLER(static, channel_extended_data_handler,
 	}
     }
   else
-    {
-      lsh_string_free(packet);
-      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_EXTENDED_DATA message.");
-    }
+    PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_EXTENDED_DATA message.");
 }
 
 DEFINE_PACKET_HANDLER(static, channel_eof_handler,
@@ -1380,8 +1324,6 @@ DEFINE_PACKET_HANDLER(static, channel_eof_handler,
       struct ssh_channel *channel = lookup_channel(connection->table,
 						   channel_number);
 
-      lsh_string_free(packet);
-
       if (channel)
 	{
 	  if (channel->flags & (CHANNEL_RECEIVED_EOF | CHANNEL_RECEIVED_CLOSE))
@@ -1423,10 +1365,7 @@ DEFINE_PACKET_HANDLER(static, channel_eof_handler,
 	}
     }
   else
-    {
-      lsh_string_free(packet);
-      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_EOF message");
-    }
+    PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_EOF message");
 }
 
 DEFINE_PACKET_HANDLER(static, channel_close_handler,
@@ -1446,8 +1385,6 @@ DEFINE_PACKET_HANDLER(static, channel_close_handler,
       struct ssh_channel *channel = lookup_channel(connection->table,
 						   channel_number);
 
-      lsh_string_free(packet);
-      
       if (channel)
 	{
 	  verbose("Receiving CLOSE on channel %i (local %i)\n",
@@ -1468,13 +1405,6 @@ DEFINE_PACKET_HANDLER(static, channel_close_handler,
 		  werror("Unexpected channel CLOSE.\n");
 		}
 
-#if 0
-	      /* FIXME: Which eof-handlers rely on being called in
-	       * this case? */
-	      if (! (channel->flags & (CHANNEL_RECEIVED_EOF))
-		  && channel->eof)
-		CHANNEL_EOF(channel);
-#endif
 	      if (channel->flags & CHANNEL_SENT_CLOSE)
 		{
 		  static const struct exception finish_exception
@@ -1495,10 +1425,7 @@ DEFINE_PACKET_HANDLER(static, channel_close_handler,
 	}
     }
   else
-    {
-      lsh_string_free(packet);
-      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_CLOSE message");
-    }
+    PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_CLOSE message");
 }
 
 DEFINE_PACKET_HANDLER(static, channel_open_confirm_handler,
@@ -1525,8 +1452,6 @@ DEFINE_PACKET_HANDLER(static, channel_open_confirm_handler,
 	lookup_channel_reserved(connection->table,
 				local_channel_number);
 
-      lsh_string_free(packet);
-
       if (channel) 
 	{
 	  struct command_continuation *c = channel->open_continuation;
@@ -1551,10 +1476,7 @@ DEFINE_PACKET_HANDLER(static, channel_open_confirm_handler,
 	}
     }
   else
-    {
-      lsh_string_free(packet);
-      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_OPEN_CONFIRMATION message.");
-    }
+    PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_OPEN_CONFIRMATION message.");
 }
 
 DEFINE_PACKET_HANDLER(static, channel_open_failure_handler,
@@ -1585,8 +1507,6 @@ DEFINE_PACKET_HANDLER(static, channel_open_failure_handler,
 	lookup_channel_reserved(connection->table,
 				channel_number);
 
-      lsh_string_free(packet); 
-
       if (channel)
 	{
 	  static const struct exception finish_exception
@@ -1604,10 +1524,7 @@ DEFINE_PACKET_HANDLER(static, channel_open_failure_handler,
 	       channel_number);
     }
   else
-    {
-      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_OPEN_FAILURE message.");
-      lsh_string_free(packet);
-    }
+    PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_OPEN_FAILURE message.");
 }
 
 DEFINE_PACKET_HANDLER(static, channel_success_handler,
@@ -1626,8 +1543,6 @@ DEFINE_PACKET_HANDLER(static, channel_success_handler,
       && parse_eod(&buffer)
       && (channel = lookup_channel(connection->table, channel_number)))
     {
-      lsh_string_free(packet);
-
       if (object_queue_is_empty(&channel->pending_requests))
 	{
 	  werror("do_channel_success: Unexpected message. Ignoring.\n");
@@ -1641,10 +1556,7 @@ DEFINE_PACKET_HANDLER(static, channel_success_handler,
 	}
     }
   else
-    {
-      lsh_string_free(packet);
-      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_SUCCESS message");
-    }
+    PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_SUCCESS message");
 }
 
 DEFINE_PACKET_HANDLER(static, channel_failure_handler,
@@ -1663,8 +1575,6 @@ DEFINE_PACKET_HANDLER(static, channel_failure_handler,
       && parse_eod(&buffer)
       && (channel = lookup_channel(connection->table, channel_number)))
     {
-      lsh_string_free(packet);
-      
       if (object_queue_is_empty(&channel->pending_requests))
 	{
 	  werror("do_channel_failure: No handler. Ignoring.\n");
@@ -1681,10 +1591,7 @@ DEFINE_PACKET_HANDLER(static, channel_failure_handler,
 	}
     }
   else
-    {
-      lsh_string_free(packet);
-      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_FAILURE message.");
-    }
+    PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_FAILURE message.");
 }
 
 
@@ -1958,7 +1865,8 @@ make_channel_read_data(struct ssh_channel *channel)
    *
    * gives 9 bytes of overhead, including the length field. */
     
-  return make_read_data(channel, make_channel_write(channel)); }
+  return make_read_data(channel, make_channel_write(channel));
+}
 
 struct io_callback *
 make_channel_read_stderr(struct ssh_channel *channel)
