@@ -39,6 +39,7 @@ struct client_userauth;
 
 #include "client_userauth.c.x"
 
+/* The last parameters says whether or not to free the password. */
 struct lsh_string *
 format_userauth_password(struct lsh_string *name,
 			 int service,
@@ -126,6 +127,7 @@ format_userauth_publickey(struct lsh_string *name,
      (name client_userauth)
      (super command)
      (vars
+       ; The name should already be converted to utf8
        (username string)            ; Remote user name to authenticate as.
        (service_name . int)         ; Service we want to access.
        (methods object object_list) ; Authentication methods, in order.
@@ -153,7 +155,7 @@ make_client_userauth_state(struct client_userauth *userauth,
 {
   NEW(client_userauth_state, self);
 
-  trace("client_userauth.c: make_client_userauth_state()\n");
+  trace("client_userauth.c: make_client_userauth_state\n");
   self->connection = connection;
   self->userauth = userauth;
   self->failure = NULL;
@@ -200,8 +202,6 @@ do_userauth_success(struct packet_handler *c,
     {
       unsigned i;
       
-      lsh_string_free(packet);
-
       verbose("User authentication successful.\n");
 
       for (i = SSH_FIRST_USERAUTH_GENERIC; i < SSH_FIRST_CONNECTION_GENERIC; i++) 
@@ -210,10 +210,7 @@ do_userauth_success(struct packet_handler *c,
       COMMAND_RETURN(self->c, connection);
     }
   else
-    {
-      lsh_string_free(packet);
-      PROTOCOL_ERROR(connection->e, "Invalid USERAUTH_SUCCESS message");
-    }
+    PROTOCOL_ERROR(connection->e, "Invalid USERAUTH_SUCCESS message");
 }
 
 static struct packet_handler *
@@ -280,8 +277,6 @@ do_userauth_failure(struct packet_handler *c,
       && parse_boolean(&buffer, &partial_success)
       && parse_eod(&buffer))
     {
-      lsh_string_free(packet);
-      
       if (partial_success)
 	/* Doesn't help us */
 	werror("Received SSH_MSH_USERAUTH_FAILURE "
@@ -297,10 +292,7 @@ do_userauth_failure(struct packet_handler *c,
 							methods, self->state->current));
     }
   else
-    {
-      lsh_string_free(packet);
-      PROTOCOL_ERROR(connection->e, "Invalid USERAUTH_FAILURE message.");
-    }
+    PROTOCOL_ERROR(connection->e, "Invalid USERAUTH_FAILURE message.");
   
   KILL(methods);	
 }
@@ -345,8 +337,6 @@ do_userauth_banner(struct packet_handler *self UNUSED,
     }
   else
     PROTOCOL_ERROR(connection->e, "Invalid USERAUTH_SUCCESS message");
-
-  lsh_string_free(packet);
 }
 
 static struct packet_handler userauth_banner_handler =
@@ -496,7 +486,7 @@ make_client_userauth(struct lsh_string *username,
   NEW(client_userauth, self);
 
   self->super.call = do_client_userauth;
-  self->username = username;
+  self->username = local_to_utf8(username, 1);
   self->service_name = service_name;
   self->methods = methods;
 
@@ -534,7 +524,7 @@ send_password(struct client_password_state *state)
       /* Password empty? */
       if (!passwd->length)
 	{
-	  /* NOTE: At least on some systems, the getpass() function
+	  /* NOTE: At least on some systems, the getpass function
 	   * sets the tty to raw mode, disabling ^C, ^D and the like.
 	   *
 	   * To be a little friendlier, we stop asking if the users
@@ -544,7 +534,7 @@ send_password(struct client_password_state *state)
 	}
 
       C_WRITE(state->connection,
-	      format_userauth_password(local_to_utf8(state->userauth->username, 0),
+	      format_userauth_password(state->userauth->username,
 				       state->userauth->service_name,
 				       local_to_utf8(passwd, 1),
 				       1));
@@ -553,7 +543,7 @@ send_password(struct client_password_state *state)
     {
       /* If the user aborts the password dialogue (by pressing ^D at
        * the terminal, or by closing a password popup window,
-       * whatever), read_password() should return NULL, and we should
+       * whatever), read_password should return NULL, and we should
        * skip to the next authentication method. */
       static const struct exception no_passwd =
 	STATIC_EXCEPTION(EXC_USERAUTH, "No password supplied.");
@@ -789,19 +779,20 @@ do_userauth_pk_ok(struct packet_handler *s,
 	{
 	  struct lsh_string *request;
 	  struct lsh_string *signed_data;
-    
+
 #if DATAFELLOWS_WORKAROUNDS
 	  if (connection->peer_flags & PEER_USERAUTH_REQUEST_KLUDGE)
-	    request = format_userauth_publickey(local_to_utf8(self->state->userauth->username, 0),
+	    request = format_userauth_publickey(self->state->userauth->username,
 						ATOM_SSH_USERAUTH,
 						key->type,
 						key->public);
 	  else
 #endif /* DATAFELLOWS_WORKAROUNDS */ 
-	    request = format_userauth_publickey(local_to_utf8(self->state->userauth->username, 0),
+	    request = format_userauth_publickey(self->state->userauth->username,
 						self->state->userauth->service_name,
 						key->type,
 						key->public);
+
 	  signed_data = ssh_format("%S%lS", connection->session_id, request);
 	  request = ssh_format("%flS%fS", 
 			       request, 
@@ -813,14 +804,10 @@ do_userauth_pk_ok(struct packet_handler *s,
       else
 	werror("client_userauth.c: Unexpected key in USERAUTH_PK_OK message.\n");
 
-      lsh_string_free(packet);
       client_publickey_next(self->state);
     }
   else
-    {
-      lsh_string_free(packet);
-      PROTOCOL_ERROR(self->state->e, "Invalid USERAUTH_PK_OK message");
-    }
+    PROTOCOL_ERROR(self->state->e, "Invalid USERAUTH_PK_OK message");
 }
 
 static struct packet_handler *
@@ -857,8 +844,10 @@ do_publickey_login(struct client_userauth_method *s,
       {
 	CAST(keypair, key, LIST(self->keys)[i]);
 
+	/* NOTE: The PEER_USERAUTH_REQUEST_KLUDGE only applies to the
+	 * signed data. */
 	C_WRITE(connection, 
-		format_userauth_publickey_query(local_to_utf8(userauth->username, 0),
+		format_userauth_publickey_query(userauth->username,
 						userauth->service_name,
 						key->type, key->public));
       }
