@@ -60,6 +60,7 @@ connection_handle_packet(struct ssh_connection *closure,
     {
       werror("connection.c: Received empty packet!\n");
       PROTOCOL_ERROR(closure->e, "Received empty packet.");
+      lsh_string_free(packet);
       return;
     }
 
@@ -67,6 +68,7 @@ connection_handle_packet(struct ssh_connection *closure,
     {
       werror("connection.c: Packet too large!\n");
       PROTOCOL_ERROR(closure->e, "Packet too large");
+      lsh_string_free(packet);
       return;
     }
     
@@ -82,6 +84,7 @@ connection_handle_packet(struct ssh_connection *closure,
 	{
 	  werror("Unexpected NEWKEYS message!\n");
 	  PROTOCOL_ERROR(closure->e, "Unexpected NEWKEYS message!");
+	  lsh_string_free(packet);
 	  return;
 	}
       break;
@@ -102,16 +105,20 @@ connection_handle_packet(struct ssh_connection *closure,
 	{
 	  werror("Unexpected KEXINIT or NEWKEYS message!\n");
 	  PROTOCOL_ERROR(closure->e, "Unexpected KEXINIT or NEWKEYS message!");
+	  lsh_string_free(packet);
 	  return;
 	}
       break;
     case KEX_STATE_NEWKEYS:
-      if ( (msg != SSH_MSG_NEWKEYS)
-	   && (msg != SSH_MSG_DISCONNECT) )
+      if (! ((msg == SSH_MSG_NEWKEYS)
+	     || (msg == SSH_MSG_DISCONNECT)
+	     || (msg == SSH_MSG_IGNORE)
+	     || (msg == SSH_MSG_DEBUG)))
 	{
 	  werror("Expected NEWKEYS message, but received message %i!\n",
 		 msg);
 	  PROTOCOL_ERROR(closure->e, "Expected NEWKEYS message");
+	  lsh_string_free(packet);
 	  return;
 	}
       break;
@@ -120,6 +127,7 @@ connection_handle_packet(struct ssh_connection *closure,
     }
 
   HANDLE_PACKET(closure->dispatch[msg], closure, packet);
+  lsh_string_free(packet);
 }
 
 static void
@@ -150,15 +158,12 @@ do_handle_connection(struct abstract_write *w,
 }
 
 DEFINE_PACKET_HANDLER(, connection_ignore_handler,
-                      connection UNUSED, packet)
+                      connection UNUSED, packet UNUSED)
 {
-  lsh_string_free(packet);
 }
 
-DEFINE_PACKET_HANDLER(, connection_fail_handler, connection, packet)
+DEFINE_PACKET_HANDLER(, connection_fail_handler, connection, packet UNUSED)
 {
-  lsh_string_free(packet);
-
   PROTOCOL_ERROR(connection->e, NULL);
 }
 
@@ -171,8 +176,6 @@ DEFINE_PACKET_HANDLER(, connection_unimplemented_handler, connection, packet)
 	  ssh_format("%c%i",
 		     SSH_MSG_UNIMPLEMENTED,
 		     packet->sequence_number));
-  
-  lsh_string_free(packet);
 }
 
 DEFINE_PACKET_HANDLER(, connection_forward_handler, connection, packet)
@@ -222,7 +225,7 @@ do_exc_connection_handler(struct exception_handler *s,
        * the peer sends us more data.
        *
        * So any code that raises EXC_PAUSE_START_CONNECTION should
-       * also call connection_handle_pending() at a safe place. We
+       * also call connection_handle_pending at a safe place. We
        * can't call it here, as we may be in the middle of the
        * handling of a packet. Installing a callout would be best. */
       
@@ -305,18 +308,6 @@ make_ssh_connection(UINT32 flags,
 
   connection->literal_kexinits[CONNECTION_CLIENT]
     = connection->literal_kexinits[CONNECTION_SERVER] = NULL;
-
-#if 0
-  connection->newkeys = NULL;
-#endif
-
-#if 0
-  /* Initialize dispatch */
-  connection->ignore = &connection_ignore_handler;
-  connection->unimplemented = &unimplemented_handler;
-  connection->fail = &fail_handler;
-  connection->forward = &forward_handler;
-#endif
   
   for (i = 0; i < 0x100; i++)
     connection->dispatch[i] = &connection_unimplemented_handler;
