@@ -2,7 +2,7 @@
  *
  * Create a packet from a format string and arguments.
  *
- * $Id: format.h,v 1.31 2000/07/28 08:00:48 nisse Exp $ */
+ * $Id: format.h,v 1.34 2001/04/26 04:43:55 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -84,33 +84,16 @@ void ssh_format_write(const char *format, UINT32 length, UINT8 *buffer, ...);
 UINT32 ssh_vformat_length(const char *format, va_list args);
 void ssh_vformat_write(const char *format, UINT32 length, UINT8 *buffer, va_list args);
 
-/* Used for generating strings that can be used with standard libc
- * functions. It checks that the resulting string doesn't contain any
- * NUL characters (if it does, NULL is returned), and adds a
- * termianting NUL character at the end, not included in the string
- * length. */
-struct lsh_string *ssh_cformat(const char *format, ...);
-
+/* FIXME: Do we need a variant that handles non-const strings? */
+/* Returns an ordinary NUL-terminated string, or NULL if the string
+ * contains any NUL-character. */
+const char *
+lsh_get_cstring(const struct lsh_string *s);
+     
 /* Short cuts */
 #define lsh_string_dup(s) (ssh_format("%lS", (s)))
 
-/* FIXME: The names of these three functions are not quite intuitive.
- * Perhaps they should be renamed or deleted. */
-
-#if 0
-#define format_cstring(s) (ssh_cformat("%lz", (s)))
-#define make_cstring_l(l, s) (ssh_cformat("%ls", (l), (s)))
-#define make_cstring(s, f) (ssh_cformat((f) ? "%lfS" : "%lS", (s)))
-#endif
-
-struct lsh_string *
-format_cstring(const char *s);
-
-struct lsh_string *
-make_cstring_l(UINT32 length, const UINT8 *data);
-
-struct lsh_string *
-make_cstring(struct lsh_string *s, int free);
+#define make_string(s) (ssh_format("%lz", (s)))
 
 unsigned format_size_in_decimal(UINT32 n);
 void format_decimal(unsigned length, UINT8 *buffer, UINT32 n);
