@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.128.2.3 2001/05/23 18:07:20 nisse Exp $ */
+ * $Id: io.c,v 1.136 2001/04/26 04:47:59 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -92,6 +92,7 @@
 /* GABA:
    (class
      (name io_backend)
+     (super resource)
      (vars
        ; Linked list of fds. 
        (files object lsh_fd)
@@ -180,7 +181,7 @@ int io_iter(struct io_backend *b)
      *
      * FIXME: How can we improve this? We could keep a stack of closed
      * files, but that will require backpointers from the fd:s to the
-     * backend (so that kill_fd can find the top of the stack). */
+     * backend (so that close_fd can find the top of the stack). */
 
     do
       {
@@ -208,7 +209,7 @@ int io_iter(struct io_backend *b)
 			LSH_CALLBACK(fd->close_callback);
 			need_close = 1;
 		      }
-		    trace("io.c: Closing fd %i.\n", fd->fd);
+		    trace("io.c: Closing fd %i: %z\n", fd->fd, fd->label);
 		
 		    if (close(fd->fd) < 0)
 		      {
@@ -278,11 +279,11 @@ int io_iter(struct io_backend *b)
 
   if (!res)
     {
-      gc_maybe(&b->super, 0);
+      gc_maybe(&b->super.super, 0);
       res = poll(fds, nfds, -1);
     }
   else
-    gc_maybe(&b->super, 1);
+    gc_maybe(&b->super.super, 1);
   
   if (!res)
     {
@@ -326,7 +327,7 @@ int io_iter(struct io_backend *b)
 	    werror("io.c: poll request on fd %i, for events of type %xi\n"
 		   "      return POLLNVAL, revents = %xi\n",
 		   fds[i].fd, fds[i].events, fds[i].revents);
-	    kill_fd(fd);
+	    close_fd(fd);
 	    continue;
 	  }
 #endif /* POLLNVAL */
@@ -418,7 +419,6 @@ int io_iter(struct io_backend *b)
 
   return 1;
 }
-
   
 void
 io_run(struct io_backend *b)
@@ -437,11 +437,60 @@ io_run(struct io_backend *b)
     ;
 }
 
+static void
+do_kill_io_backend(struct resource *s)
+{
+  CAST(io_backend, backend, s);
+
+  if (backend->super.alive)
+    {
+      struct lsh_fd *fd;
+      struct lsh_signal_handler *signal;
+  
+      for (fd = backend->files, backend->files = NULL;
+	   fd; fd = fd->next)
+	{
+	  close_fd(fd);
+      
+	  if (fd->fd < 0)
+	    /* Unlink silently. */
+	    ;
+	  else
+	    {
+	      if (fd->write_close)
+		FD_WRITE_CLOSE(fd);
+	  
+	      if (fd->close_callback)
+		LSH_CALLBACK(fd->close_callback);
+	  
+	      if (close(fd->fd) < 0)
+		{
+		  werror("io.c: close failed, (errno = %i): %z\n",
+			 errno, STRERROR(errno));
+		  EXCEPTION_RAISE(fd->e,
+				  make_io_exception(EXC_IO_CLOSE, fd,
+						    errno, NULL));
+		}
+	    }
+	}
+      /* Check that no callback has opened new files. */
+      assert(!backend->files);
+
+      for (signal = backend->signals, backend->signals = NULL;
+	   signal; signal = signal->next)
+	signal->super.alive = 0;
+
+      backend->super.alive = 0;
+    }
+}
+
 struct io_backend *
 make_io_backend(void)
 {
   NEW(io_backend, b);
 
+  resource_init(&b->super, do_kill_io_backend);
+  
   b->files = NULL;
   b->signals = NULL;
   b->callouts = NULL;
@@ -449,6 +498,12 @@ make_io_backend(void)
   return b;
 }
 
+void
+io_final(struct io_backend *b)
+{
+  KILL_RESOURCE(&b->super);
+}
+
 struct resource *
 io_signal_handler(struct io_backend *b,
 		  volatile sig_atomic_t *flag,
@@ -513,6 +568,9 @@ do_buffered_read(struct io_callback *s,
 	EXCEPTION_RAISE(fd->e, 
 			make_io_exception(EXC_IO_READ, fd,
 					  errno, NULL));
+	/* Close the fd, unless it has a write callback. */
+	close_fd_read(fd);
+	
 	break;
       }
   else if (res > 0)
@@ -619,24 +677,27 @@ do_consuming_read(struct io_callback *c,
       int res = read(fd->fd, s->data, wanted);
 
       if (res < 0)
-	switch(errno)
-	  {
-	  case EINTR:
-	    break;
-	  case EWOULDBLOCK:
-	    werror("io.c: read_consume: Unexpected EWOULDBLOCK\n");
-	    break;
-	  case EPIPE:
-	    /* FIXME: I don't understand why reading should return
-	     * EPIPE, but it happens occasionally under linux. Perhaps
-	     * we should treat it as EOF instead? */
-	    werror("io.c: read_consume: Unexpected EPIPE.\n");
-	  default:
-	    EXCEPTION_RAISE(fd->e, 
-			    make_io_exception(EXC_IO_READ,
-					      fd, errno, NULL));
-	    break;
-	  }
+	{
+	  switch(errno)
+	    {
+	    case EINTR:
+	      break;
+	    case EWOULDBLOCK:
+	      werror("io.c: read_consume: Unexpected EWOULDBLOCK\n");
+	      break;
+	    case EPIPE:
+	      /* FIXME: I don't understand why reading should return
+	       * EPIPE, but it happens occasionally under linux. Perhaps
+	       * we should treat it as EOF instead? */
+	      werror("io.c: read_consume: Unexpected EPIPE.\n");
+	    default:
+	      EXCEPTION_RAISE(fd->e, 
+			      make_io_exception(EXC_IO_READ,
+						fd, errno, NULL));
+	      break;
+	    }
+	  lsh_string_free(s);
+	}
       else if (res > 0)
 	{
 	  s->length = res;
@@ -644,6 +705,7 @@ do_consuming_read(struct io_callback *c,
 	}
       else
 	{
+	  lsh_string_free(s);
 	eof:
 	  /* Close the fd, unless it has a write callback. */
 	  close_fd_read(fd);
@@ -688,8 +750,8 @@ do_write_callback(struct io_callback *s UNUSED,
 	break;
       case EPIPE:
 	debug("io.c: Broken pipe.\n");
-	close_fd(fd);
-	break;
+	
+	/* Fall through */
       default:
 	werror("io.c: write failed, %z\n", STRERROR(errno));
 	EXCEPTION_RAISE(fd->e,
@@ -775,7 +837,7 @@ do_listen_callback(struct io_callback *s,
   trace("io.c: accept on fd %i\n", conn);
   COMMAND_RETURN(self->c,
 		 make_listen_value(make_lsh_fd(self->backend,
-					       conn, self->e),
+					       conn, "accepted socket", self->e),
 				   sockaddr2info(addr_len,
 						 (struct sockaddr *) &peer)));
 }
@@ -794,41 +856,6 @@ make_listen_callback(struct io_backend *backend,
   return &self->super;
 }
 
-#if 0
-
-static void
-do_listen_callback_no_peer(struct io_callback *s,
-			   struct lsh_fd *fd)
-{
-  CAST(io_listen_callback, self, s);
-
-  int conn;
-
-  conn = accept(fd->fd,
-		(struct sockaddr *) &peer, &addr_len);
-  if (conn < 0)
-    {
-      werror("io.c: accept failed, %z", STRERROR(errno));
-      return;
-    }
-  trace("io.c: accept on fd %i\n", conn);
-  COMMAND_RETURN(self->c, make_lsh_fd(self->backend,
-				      conn, self->e));
-}
-
-struct io_callback *
-make_listen_callback_no_peer(struct io_backend *backend,
-			     struct command_continuation *c)
-{
-  NEW(io_listen_callback, self);
-  self->super.f = do_listen_callback_no_peer;
-  self->backend = backend;
-  self->c = c;
-  
-  return &self->super;
-}
-#endif
-
 /* Connect callback */
 
 /* GABA:
@@ -855,12 +882,13 @@ do_connect_callback(struct io_callback *s,
       debug("io.c: connect_callback: Connect failed.\n");
       EXCEPTION_RAISE(fd->e,
 		      make_io_exception(EXC_IO_CONNECT, fd, 0, "connect failed."));
-      kill_fd(fd);
+      close_fd(fd);
     }
   else
     {
       fd->write = NULL;
       fd->want_write = 0;
+      fd->label = "connected socket";
       COMMAND_RETURN(self->c, fd);
     }
 }
@@ -912,12 +940,14 @@ do_exc_io_handler(struct exception_handler *self,
 /* Initializes a file structure, and adds it to the backend's list. */
 static void
 init_file(struct io_backend *b, struct lsh_fd *f, int fd,
+	  const char *label,
 	  struct exception_handler *e)
 {
   resource_init(&f->super, do_kill_fd);
 
   f->fd = fd;
-
+  f->label = label;
+  
   f->e = make_exception_handler(do_exc_io_handler, e, HANDLER_CONTEXT);
   
   f->close_callback = NULL;
@@ -1159,8 +1189,7 @@ sockaddr2info(size_t addr_len UNUSED,
 #if HAVE_GETADDRINFO
 static struct addrinfo *
 choose_address(struct addrinfo *list,
-	       const int *preference,
-	       int *s)
+	       const int *preference)
 {
   int i;
   for (i = 0; preference[i]; i++)
@@ -1168,14 +1197,7 @@ choose_address(struct addrinfo *list,
       struct addrinfo *p;
       for (p = list; p; p = p->ai_next)
 	if (preference[i] == p->ai_family)
-	  {
-	    *s = socket(p->ai_family, p->ai_socktype, p->ai_protocol);
-	    if (*s<0)
-	      continue;
-
-	    debug("io.c: address of type %i chosen.\n", p->ai_family);
-	    return p;
-	  }
+	  return p;
     }
   return NULL;
 }
@@ -1186,7 +1208,8 @@ choose_address(struct addrinfo *list,
 struct sockaddr *
 address_info2sockaddr(socklen_t *length,
 		      struct address_info *a,
-		      int *s,
+		      /* Preferred address families. Zero-terminated array. */
+		      const int *preference,
 		      int lookup)
 {
   char *host;
@@ -1220,7 +1243,7 @@ address_info2sockaddr(socklen_t *length,
     int err;
     /* FIXME: It seems ugly to have to convert the port number to a
      * string. */
-    struct lsh_string *service = ssh_cformat("%di", a->port);
+    struct lsh_string *service = ssh_format("%di", a->port);
 
     memset(&hints, 0, sizeof(hints));
     hints.ai_family = PF_UNSPEC;
@@ -1230,7 +1253,7 @@ address_info2sockaddr(socklen_t *length,
     if (!lookup)
       hints.ai_flags |= AI_NUMERICHOST;
     
-    err = getaddrinfo(host, service->data, &hints, &list);
+    err = getaddrinfo(host, lsh_get_cstring(service), &hints, &list);
     lsh_string_free(service);
 
     if (err)
@@ -1241,7 +1264,7 @@ address_info2sockaddr(socklen_t *length,
       }
 
     chosen = choose_address(list,
-			    default_preference, s);
+			    preference ? preference : default_preference);
     if (!chosen)
       {
 	freeaddrinfo(list);
@@ -1314,15 +1337,6 @@ address_info2sockaddr(socklen_t *length,
 	      memcpy(&addr->sin_addr, hp->h_addr, hp->h_length);
 	    }
 	}
-      if (addr)
-	{
-	  *s = socket(AF_INET, SOCK_STREAM, 0);
-	  if (*s < 0)
-	    {
-	      lsh_space_free(addr);
-	      addr = NULL;
-	    }
-	}
       return (struct sockaddr *) addr;
     }
 #endif /* !HAVE_GETADDRINFO */  
@@ -1378,13 +1392,13 @@ void io_init_fd(int fd)
 
 struct lsh_fd *
 make_lsh_fd(struct io_backend *b,
-	    int fd,
+	    int fd, const char *label,
 	    struct exception_handler *e)
 {
   NEW(lsh_fd, f);
 
   io_init_fd(fd);
-  init_file(b, f, fd, e);
+  init_file(b, f, fd, label, e);
 
   return f;
 }
@@ -1392,15 +1406,16 @@ make_lsh_fd(struct io_backend *b,
 /* Some code is taken from Thomas Bellman's tcputils. */
 struct lsh_fd *
 io_connect(struct io_backend *b,
-	   int s,
 	   struct sockaddr *remote,
 	   socklen_t remote_length,
 	   struct command_continuation *c,
 	   struct exception_handler *e)
 {
+  int s = socket(remote->sa_family, SOCK_STREAM, 0);
   struct lsh_fd *fd;
   
-  assert(s >= 0);
+  if (s<0)
+    return NULL;
 
   trace("io.c: Connecting using fd %i\n", s);
   
@@ -1425,7 +1440,7 @@ io_connect(struct io_backend *b,
       return NULL;
     }
 
-  fd = make_lsh_fd(b, s, e);
+  fd = make_lsh_fd(b, s, "connecting socket", e);
   
   fd->want_write = 1;
   fd->write = make_connect_callback(c);
@@ -1435,15 +1450,16 @@ io_connect(struct io_backend *b,
 
 struct lsh_fd *
 io_listen(struct io_backend *b,
-	  int s,
 	  struct sockaddr *local,
 	  socklen_t length,
 	  struct io_callback *callback,
 	  struct exception_handler *e)
 {
+  int s = socket(local->sa_family, SOCK_STREAM, 0);
   struct lsh_fd *fd;
-
-  assert(s >= 0);
+  
+  if (s<0)
+    return NULL;
 
   trace("io.c: Listening on fd %i\n", s);
   
@@ -1466,7 +1482,7 @@ io_listen(struct io_backend *b,
       return NULL;
     }
 
-  fd = make_lsh_fd(b, s, e);
+  fd = make_lsh_fd(b, s, "listening socket", e);
 
   fd->want_read = 1;
   fd->read = callback;
@@ -1486,8 +1502,8 @@ make_local_info(struct lsh_string *directory,
   if (!directory || !name || memchr(name->data, '/', name->length))
     return NULL;
 
-  assert(NUL_TERMINATED(directory));
-  assert(NUL_TERMINATED(name));
+  assert(lsh_get_cstring(directory));
+  assert(lsh_get_cstring(name));
   
   {
     NEW(local_info, self);
@@ -1601,11 +1617,13 @@ io_listen_local(struct io_backend *b,
   struct sockaddr_un *local;
   socklen_t local_length;
 
-  int s;
   struct lsh_fd *fd;
+
+  const char *cdir = lsh_get_cstring(info->directory);
+  const char *cname = lsh_get_cstring(info->name);
   
-  assert(info->directory && NUL_TERMINATED(info->directory));
-  assert(info->name && NUL_TERMINATED(info->name));
+  assert(cdir);
+  assert(cname);
 
   /* NAME should not be a plain filename, with no directory separators.
    * In particular, it should not be an absolute filename. */
@@ -1619,7 +1637,7 @@ io_listen_local(struct io_backend *b,
 
   /* cd to it, but first save old cwd */
 
-  old_cd = safe_pushd(info->directory->data, 1);
+  old_cd = safe_pushd(cdir, 1);
   if (old_cd < 0)
     return NULL;
 
@@ -1627,12 +1645,12 @@ io_listen_local(struct io_backend *b,
    * creating a socket. */
 
   /* Try unlinking any existing file. */
-  if ( (unlink(info->name->data) < 0)
+  if ( (unlink(cname) < 0)
        && (errno != ENOENT))
     {
       werror("io.c: unlink '%S'/'%S' failed (errno = %i): %z\n",
 	     info->directory, info->name, errno, STRERROR(errno));
-      safe_popd(old_cd, info->directory->data);
+      safe_popd(old_cd, cdir);
       return NULL;
     }
 
@@ -1642,10 +1660,7 @@ io_listen_local(struct io_backend *b,
   old_umask = umask(0077);
 
   /* Bind and listen */
-  s = socket(AF_UNIX, SOCK_STREAM, 0);
-  fd = ( (s >= 0)
-	 ? io_listen(b, s, (struct sockaddr *) local, local_length, callback, e)
-	 : NULL);
+  fd = io_listen(b, (struct sockaddr *) local, local_length, callback, e);
   
   /* Ok, now we restore umask and cwd */
   umask(old_umask);
@@ -1667,11 +1682,12 @@ io_connect_local(struct io_backend *b,
   struct sockaddr_un *addr;
   socklen_t addr_length;
 
-  int s;
   struct lsh_fd *fd;
   
-  assert(info->directory && NUL_TERMINATED(info->directory));
-  assert(info->name && NUL_TERMINATED(info->name));
+  const char *cdir = lsh_get_cstring(info->directory);
+  
+  assert(cdir);
+  assert(lsh_get_cstring(info->name));
 
   /* NAME should not be a plain filename, with no directory separators.
    * In particular, it should not be an absolute filename. */
@@ -1685,16 +1701,13 @@ io_connect_local(struct io_backend *b,
 
   /* cd to it, but first save old cwd */
 
-  old_cd = safe_pushd(info->directory->data, 0);
+  old_cd = safe_pushd(cdir, 0);
   if (old_cd < 0)
     return NULL;
+  
+  fd = io_connect(b, (struct sockaddr *) addr, addr_length, c, e);
 
-  s = socket(AF_UNIX, SOCK_STREAM, 0);
-  fd = ( (s >= 0)
-	 ? io_connect(b, s, (struct sockaddr *) addr, addr_length, c, e)
-	 : NULL);
-
-  safe_popd(old_cd, info->directory->data);
+  safe_popd(old_cd, cdir);
 
   return fd;
 }
@@ -1715,7 +1728,7 @@ io_read_write(struct lsh_fd *fd,
   fd->want_read = !!read;
   
   /* Writing */
-  fd->write_buffer = write_buffer_alloc(block_size);
+  fd->write_buffer = make_write_buffer(block_size);
   fd->write = &io_write_callback;
 
   fd->prepare = do_write_prepare;
@@ -1751,7 +1764,7 @@ io_write(struct lsh_fd *fd,
   trace("io.c: Preparing fd %i for writing\n", fd->fd);
   
   /* Writing */
-  fd->write_buffer = write_buffer_alloc(block_size);
+  fd->write_buffer = make_write_buffer(block_size);
   fd->write = &io_write_callback;
 
   fd->prepare = do_write_prepare;
@@ -1773,7 +1786,8 @@ io_write_file(struct io_backend *backend,
   if (fd < 0)
     return NULL;
 
-  return io_write(make_lsh_fd(backend, fd, e), block_size, c);
+  return io_write(make_lsh_fd(backend, fd, "write-only file", e),
+		  block_size, c);
 }
 
 struct lsh_fd *
@@ -1785,24 +1799,23 @@ io_read_file(struct io_backend *backend,
   if (fd < 0)
     return NULL;
 
-  return make_lsh_fd(backend, fd, e);
-}
-
-void kill_fd(struct lsh_fd *fd)
-{
-  fd->super.alive = 0;
+  return make_lsh_fd(backend, fd, "read-only file", e);
 }
 
 void close_fd(struct lsh_fd *fd)
 {
-  trace("io.c: Marking fd %i for closing.\n", fd->fd);
-  kill_fd(fd);
+  trace("io.c: Marking fd %i: %z, for closing.\n",
+	fd->fd, fd->label);
+  fd->super.alive = 0;
 }
 
 void close_fd_nicely(struct lsh_fd *fd)
 {
   /* Don't attempt to read any further. */
 
+  trace("io.c: close_fd_nicely called on fd %i: %z\n",
+	fd->fd, fd->label);
+  
   fd->want_read = 0;
   fd->read = NULL;
   
@@ -1811,7 +1824,7 @@ void close_fd_nicely(struct lsh_fd *fd)
     FD_WRITE_CLOSE(fd);
   else
     /* There's no data buffered for write. */
-    kill_fd(fd);
+    close_fd(fd);
 }
 
 /* Stop reading, but if the fd has a write callback, keep it open. */
@@ -1822,7 +1835,7 @@ void close_fd_read(struct lsh_fd *fd)
   
   if (!fd->write)
     /* We won't be writing anything on this fd, so close it. */
-    kill_fd(fd);
+    close_fd(fd);
 }
 
 /* Responsible for handling the EXC_FINISH_READ exception. It should
