@@ -2,7 +2,7 @@
  *
  * Reads an sexp in given form, and writes it in ssh2 form.
  *
- * $Id: lsh-export-key.c,v 1.5 2000/11/15 17:01:41 nisse Exp $ */
+ * $Id: lsh-export-key.c,v 1.7 2001/04/26 04:56:50 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -354,6 +354,8 @@ int appl_main(int argc, char **argv);
 #define main appl_main
 #endif
 
+#define MAX_KEY_SIZE 10000
+
 int main(int argc, char **argv)
 {
   struct export_key_options *options = make_options();
@@ -380,7 +382,7 @@ int main(int argc, char **argv)
 	}
     }
   else
-    in = make_lsh_fd(backend, STDIN_FILENO, e);
+    in = make_lsh_fd(backend, STDIN_FILENO, "stdin", e);
       
   if (options->outfile)
     {
@@ -389,13 +391,14 @@ int main(int argc, char **argv)
 			  SEXP_BUFFER_SIZE, NULL, e);
     }
   else
-    out = io_write(make_lsh_fd(backend, STDOUT_FILENO, e),
+    out = io_write(make_lsh_fd(backend, STDOUT_FILENO,
+			       "stdout", e),
 		   SEXP_BUFFER_SIZE, NULL);
 
   {
     CAST_SUBTYPE(command, work,
 		 make_export_key(
-		   make_read_sexp_command(options->input, 0),
+		   make_read_sexp_command(options->input, 0, MAX_KEY_SIZE),
 		   options->print,
 		   &(out->write_buffer->super)));
 
