@@ -1,6 +1,6 @@
 /* server_session.c
  *
- * $Id: server_session.c,v 1.59 2001/03/12 21:41:15 nisse Exp $
+ * $Id: server_session.c,v 1.64 2001/04/26 05:02:34 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -65,7 +65,7 @@
        (initial_window . UINT32)
 
        ; Resource to kill when the channel is closed. 
-       (process object resource)
+       (process object lsh_process)
 
        ; pty
        (pty object pty_info)
@@ -225,7 +225,7 @@ format_exit_signal(struct ssh_channel *channel,
   return format_channel_request(ATOM_EXIT_SIGNAL,
 				channel,
 				0,
-				"%i%c%fS%z",
+				"%a%c%fS%z",
 				signal_local_to_network(signal),
 				core,
 				msg, "");
@@ -435,7 +435,7 @@ spawn_process(struct server_session *session,
     return -1;
 
   {
-    struct resource *child;
+    struct lsh_process *child;
     
     if (USER_FORK(user, &child,
 		  make_exit_shell(session),
@@ -466,7 +466,7 @@ spawn_process(struct server_session *session,
 		= make_channel_read_close_callback(channel);
 
 	      session->in
-		= io_write(make_lsh_fd(backend, in[1],
+		= io_write(make_lsh_fd(backend, in[1], "child stdin",
 				       io_exception_handler),
 			   SSH_MAX_PACKET, NULL);
 	  
@@ -477,12 +477,14 @@ spawn_process(struct server_session *session,
 	       * which will close the channel on read errors, or is it
 	       * better to just send EOF on read errors? */
 	      session->out
-		= io_read(make_lsh_fd(backend, out[0], io_exception_handler),
+		= io_read(make_lsh_fd(backend, out[0], "child stdout",
+				      io_exception_handler),
 			  make_channel_read_data(channel),
 			  read_close_callback);
 	      session->err 
 		= ( (err[0] != -1)
-		    ? io_read(make_lsh_fd(backend, err[0], io_exception_handler),
+		    ? io_read(make_lsh_fd(backend, err[0], "child stderr",
+					  io_exception_handler),
 			      make_channel_read_stderr(channel),
 			      read_close_callback)
 		    : NULL);
@@ -495,7 +497,7 @@ spawn_process(struct server_session *session,
 	    /* Make sure that the process and it's stdio is
 	     * cleaned up if the channel or connection dies. */
 	    REMEMBER_RESOURCE
-	      (channel->resources, child);
+	      (channel->resources, &child->super);
 
 	    /* FIXME: How to do this properly if in and out may use the
 	     * same fd? */
@@ -637,7 +639,7 @@ do_spawn_shell(struct channel_request *c,
       { /* Child */
 #define MAX_ENV 1
 	/* No args, end the USER_EXEC method fills in argv[0]. */
-	char *argv[] = { NULL, NULL };
+	const char *argv[] = { NULL, NULL };
 	
 	struct env_value env[MAX_ENV];
 	int env_length = 0;
@@ -713,8 +715,6 @@ do_spawn_exec(struct channel_request *c,
   UINT32 command_len;
   const UINT8 *command;
 
-  struct lsh_string *command_line;
-  
   if (!(parse_string(args, &command_len, &command)
 	&& parse_eod(args)))
     {
@@ -722,73 +722,74 @@ do_spawn_exec(struct channel_request *c,
       return;
     }
     
-  if (session->process)
-    /* Already spawned a shell or command */
-    goto fail;
-
-  command_line = ssh_cformat("%ls", command_len, command);
-  
-  if (!command_line)
+  if (/* Already spawned a shell or command */
+      session->process
+      /* Command can't contain NUL characters. */
+      || memchr(command, '\0', command_len))
+    
     EXCEPTION_RAISE(e, &exec_request_failed);
   else
-    switch (spawn_process(session, connection->user,
-			  connection->peer,
-			  closure->backend))
     {
-    case 1: /* Parent */
-      lsh_string_free(command_line);
-
-      /* NOTE: The return value is not used. */
-      COMMAND_RETURN(s, channel);
-      channel_start_receive(channel, session->initial_window);
-      return;
-    case 0:
-      { /* Child */
+      struct lsh_string *command_line = ssh_format("%ls", command_len, command);
+      
+      switch (spawn_process(session, connection->user,
+			    connection->peer,
+			    closure->backend))
+	{
+	case 1: /* Parent */
+	  lsh_string_free(command_line);
+	  
+	  /* NOTE: The return value is not used. */
+	  COMMAND_RETURN(s, channel);
+	  channel_start_receive(channel, session->initial_window);
+	  return;
+	case 0:
+	  { /* Child */
 #define MAX_ENV 1
-	struct env_value env[MAX_ENV];
-	int env_length = 0;
+	    struct env_value env[MAX_ENV];
+	    int env_length = 0;
+	    
+	    /* No args, and the USER_EXEC method fills in argv[0]. */
 
-	/* No args, and the USER_EXEC method fills in argv[0]. */
-#if 0
-	/* NOTE: HPUX compiler can't handle array initialization. */
-	char *argv[] = { NULL, "-c", command_line->data, NULL };
-#else
-	char *argv[4];
-	argv[0] = NULL;
-	argv[1] = "-c";
-	argv[2] = command_line->data;
-	argv[3] = NULL;
-#endif
+	    /* NOTE: I'd like to use an array initializer, but that's
+	     * not ANSI-C, and at least HPUX' compiler can't handle
+	     * it. */
+	    
+	    const char *argv[4];
+	    argv[0] = NULL;
+	    argv[1] = "-c";
+	    argv[2] = lsh_get_cstring(command_line);
+	    argv[3] = NULL;
 	
-	debug("do_spawn_shell: Child process\n");
-	assert(getuid() == connection->user->uid);
+	    debug("do_spawn_shell: Child process\n");
+	    assert(getuid() == connection->user->uid);
 	    	    
-	if (session->term)
-	  {
-	    env[env_length].name ="TERM";
-	    env[env_length].value = session->term;
-	    env_length++;
-	  }
-	assert(env_length <= MAX_ENV);
+	    if (session->term)
+	      {
+		env[env_length].name ="TERM";
+		env[env_length].value = session->term;
+		env_length++;
+	      }
+	    assert(env_length <= MAX_ENV);
 #undef MAX_ENV
 
-	USER_EXEC(connection->user, 0, argv, env_length, env);
+	    USER_EXEC(connection->user, 0, argv, env_length, env);
 	
-	/* exec failed! */
-	verbose("server_session: exec failed (errno = %i): %z\n",
-		errno, STRERROR(errno));
-	_exit(EXIT_FAILURE);
-      }
-    case -1:
-      /* fork failed */
-      lsh_string_free(command_line);
+	    /* exec failed! */
+	    verbose("server_session: exec failed (errno = %i): %z\n",
+		    errno, STRERROR(errno));
+	    _exit(EXIT_FAILURE);
+	  }
+	case -1:
+	  /* fork failed */
+	  lsh_string_free(command_line);
+	  EXCEPTION_RAISE(e, &exec_request_failed);
 
-      break;
-    default:
-      fatal("Internal error!");
-  }
- fail:
-  EXCEPTION_RAISE(e, &shell_request_failed);
+	  break;
+	default:
+	  fatal("Internal error!");
+	}
+    }
 }
 
 struct channel_request *
@@ -804,9 +805,6 @@ make_exec_handler(struct io_backend *backend)
 
 #if WITH_PTY_SUPPORT
 
-static struct exception pty_request_failed =
-STATIC_EXCEPTION(EXC_CHANNEL_REQUEST, "pty request failed");
-
 /* pty_handler */
 static void
 do_alloc_pty(struct channel_request *c UNUSED,
@@ -817,74 +815,132 @@ do_alloc_pty(struct channel_request *c UNUSED,
 	     struct command_continuation *s,
 	     struct exception_handler *e)
 {
-  UINT32 width, height, width_p, height_p;
+  struct terminal_dimensions dims;
   const UINT8 *mode;
   UINT32 mode_length;
   struct lsh_string *term = NULL;
 
+  static struct exception pty_request_failed =
+    STATIC_EXCEPTION(EXC_CHANNEL_REQUEST, "pty request failed");
+
   CAST(server_session, session, channel);
 
   verbose("Client requesting a tty...\n");
-  
-  /* The client may only request a tty once. */
-  if (!session->pty &&
-      (term = parse_string_copy(args)) &&
-      parse_uint32(args, &width) &&
-      parse_uint32(args, &height) &&
-      parse_uint32(args, &width_p) &&
-      parse_uint32(args, &height_p) &&
-      parse_string(args, &mode_length, &mode) &&
-      parse_eod(args))
+
+  if ((term = parse_string_copy(args))
+      && parse_uint32(args, &dims.char_width)
+      && parse_uint32(args, &dims.char_height)
+      && parse_uint32(args, &dims.pixel_width)
+      && parse_uint32(args, &dims.pixel_height)
+      && parse_string(args, &mode_length, &mode)
+      && parse_eod(args))
     {
-      struct pty_info *pty = make_pty_info();
-
-      if (pty_allocate(pty, connection->user->uid))
-        {
-          struct termios ios;
-
-          if (tty_getattr(pty->slave, &ios))
-            {
-	      assert(pty->super.alive);
-              session->pty = pty;
-
-	      /* Don't set TERM if the value is empty. */
-	      if (!term->length)
-		{
-		  lsh_string_free(term);
-		  term = NULL;
-		}
+      /* The client may only request a tty once. */
+      if (session->pty)
+	{
+	  lsh_string_free(term);
+	  EXCEPTION_RAISE(e, &pty_request_failed);
+	}
+      else
+	{
+	  struct pty_info *pty = make_pty_info();
+      
+	  if (pty_allocate(pty, connection->user->uid))
+	    {
+	      struct termios ios;
 	      
-              session->term = term;
-              tty_decode_term_mode(&ios, mode_length, mode); 
-	      
-	      /* cfmakeraw(&ios); */
-              if (tty_setattr(pty->slave, &ios) &&
-                  tty_setwinsize(pty->slave,
-				 width, height, width_p, height_p))
+	      if (tty_getattr(pty->slave, &ios))
 		{
-		  REMEMBER_RESOURCE(channel->resources, &pty->super);
-
-		  verbose(" granted.\n");
-		  COMMAND_RETURN(s, NULL);
-
-		  return;
+		  assert(pty->super.alive);
+		  session->pty = pty;
+		  
+		  /* Don't set TERM if the value is empty. */
+		  if (!term->length)
+		    {
+		      lsh_string_free(term);
+		      term = NULL;
+		    }
+	      
+		  session->term = term;
+		  tty_decode_term_mode(&ios, mode_length, mode); 
+	      
+		  if (tty_setattr(pty->slave, &ios) &&
+		      tty_setwinsize(pty->slave, &dims))
+		    {
+		      REMEMBER_RESOURCE(channel->resources, &pty->super);
+		      
+		      verbose(" granted.\n");
+		      COMMAND_RETURN(s, NULL);
+		      
+		      return;
+		    }
 		}
-	      else
-		/* Close fd:s and mark the pty-struct as dead */
-		KILL_RESOURCE(&pty->super);
-            }
-        }
-      KILL(pty);
+	    }
+	  
+	  /* Close fd:s and mark the pty-struct as dead */
+	  KILL_RESOURCE(&pty->super);
+	  KILL(pty);
+	}
+      verbose("Pty allocation failed.\n");
+      lsh_string_free(term);
+      EXCEPTION_RAISE(e, &pty_request_failed);
     }
+  else
+    {
+      werror("Invalid pty request.\n");
+      lsh_string_free(term);
 
-  verbose("Pty allocation failed.\n");
-  lsh_string_free(term);
-
-  EXCEPTION_RAISE(e, &pty_request_failed);
+      PROTOCOL_ERROR(e, "Invalid pty request.");
+    }
 }
 
 struct channel_request
 pty_request_handler =
 { STATIC_HEADER, do_alloc_pty };
 
+static void
+do_window_change_request(struct channel_request *c UNUSED,
+			 struct ssh_channel *channel,
+			 struct ssh_connection *connection UNUSED,
+			 struct channel_request_info *info UNUSED,
+			 struct simple_buffer *args,
+			 struct command_continuation *s,
+			 struct exception_handler *e)
+{
+  struct terminal_dimensions dims;
+  CAST(server_session, session, channel);
+
+  verbose("Receiving window-change request...\n");
+
+  if (parse_uint32(args, &dims.char_width)
+      && parse_uint32(args, &dims.char_height)
+      && parse_uint32(args, &dims.pixel_width)
+      && parse_uint32(args, &dims.pixel_height)
+      && parse_eod(args))
+    {
+      static const struct exception winch_request_failed =
+	STATIC_EXCEPTION(EXC_CHANNEL_REQUEST, "window-change request failed: No pty");
+
+      if (!(session->pty && session->pty->super.alive))
+	EXCEPTION_RAISE(e, &winch_request_failed);
+      
+      else if (!tty_setwinsize(session->pty->slave, &dims))
+	EXCEPTION_RAISE(e, &winch_request_failed);
+
+      else
+	{
+	  if (!SIGNAL_PROCESS(session->process, SIGWINCH))
+	    werror("Sending SIGWINCH signal failed.\n");
+
+	  COMMAND_RETURN(s, NULL);
+	}
+    }
+  else
+    PROTOCOL_ERROR(e, "Invalid window-change request.");
+}
+
+struct channel_request
+window_change_request_handler =
+{ STATIC_HEADER, do_window_change_request };
+
 #endif /* WITH_PTY_SUPPORT */
