@@ -1,6 +1,6 @@
 /* server_userauth.h
  *
- * $Id: server_userauth.h,v 1.15 2000/10/15 20:42:23 nisse Exp $
+ * $Id: server_userauth.h,v 1.17 2001/04/08 14:51:17 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -46,6 +46,7 @@
    (class
      (name user_db)
      (vars
+       ;; FIXME: Do we really need the FREE parameter?
        (lookup method "struct lsh_user *"
                       "struct lsh_string *name" "int free")))
 */
@@ -54,7 +55,8 @@
 
 struct user_db *
 make_unix_user_db(struct io_backend *backend, struct reap *reaper,
-		  const char *pw_helper, int allow_root);
+		  const char *pw_helper, const char *login_shell,
+		  int allow_root);
 
 /* GABA:
    (class
