@@ -2,7 +2,7 @@
  *
  * SPKI interface
  *
- * $Id: spki_commands.c,v 1.18 2001/02/25 22:38:21 nisse Exp $ */
+ * $Id: spki_commands.c,v 1.21 2001/05/01 18:35:57 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -233,7 +233,7 @@ parse_private_key(struct alist *algorithms,
       break;
       
     default:
-      abort();
+      fatal("Internal error!\n");
 #if 0      
       /* Get a corresponding public key. */
       COMMAND_RETURN(c, make_keypair
@@ -330,7 +330,6 @@ DEFINE_COMMAND_SIMPLE(spki_make_context_command, a)
   return &make_spki_context(algorithms)->super;
 }
 
-
 /* Reads a file of ACL:s, and returns an spki_context. */
 
 /* GABA:
@@ -361,14 +360,7 @@ make_spki_read_acls(struct alist *algorithms)
   return res;
 }
 
-DEFINE_COMMAND_SIMPLE(spki_read_acls_command, a)
-{
-  CAST_SUBTYPE(alist, algorithms, a);
-  CAST_SUBTYPE(command, res, spki_read_acl(algorithms));
 
-  return &res->super;
-}
-  
 /* Reading of host-keys
  * ********************/
 
@@ -678,16 +670,16 @@ do_spki_decrypt(struct command *s,
 
   else
     {
-      struct lsh_string *label;
+      const struct lsh_string *label;
       struct sexp *key_info;
       struct sexp *payload;
 
       struct crypto_algorithm *crypto;
       struct mac_algorithm *mac;
       
-      struct lsh_string *salt;
-      struct lsh_string *iv;
-      struct lsh_string *data;
+      const struct lsh_string *salt;
+      const struct lsh_string *iv;
+      const struct lsh_string *data;
       UINT32 iterations;
       
       if (SEXP_LEFT(i) != 3)
