@@ -2,7 +2,7 @@
  *
  * Thomas Wu's Secure Remote Password Protocol
  *
- * $Id: srp_exchange.c,v 1.7 2000/06/04 19:59:19 nisse Exp $ */
+ * $Id: srp_exchange.c,v 1.10 2001/05/01 18:37:17 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -59,7 +59,7 @@ make_srp_entry(struct lsh_string *name, struct sexp *e)
       && sexp_atom_eq(SEXP_GET(i), ATOM_SSH_RING1) )
     {
       NEW(srp_entry, res);
-      struct lsh_string *salt;
+      const struct lsh_string *salt;
 
       mpz_init(res->verifier);
       
@@ -75,7 +75,8 @@ make_srp_entry(struct lsh_string *name, struct sexp *e)
 
       SEXP_NEXT(i);
 
-      if (!sexp2bignum_u(SEXP_GET(i), res->verifier))
+      /* FIXME: Pass a more restrictive limit to sexp2bignum_u. */
+      if (!sexp2bignum_u(SEXP_GET(i), res->verifier, 0))
 	{
 	  KILL(res);
 	  return NULL;
@@ -139,8 +140,10 @@ srp_format_proofs(struct dh_instance *dh,
 		  struct lsh_string **m1,
 		  struct lsh_string **m2)
 {
+  struct mac_algorithm *mac_algorithm
+    = make_hmac_algorithm(dh->method->H);
   struct mac_instance *hmac
-    = MAKE_MAC(make_hmac_algorithm(dh->method->H),
+    = MAKE_MAC(mac_algorithm,
 	       dh->K->length, dh->K->data);
   struct lsh_string *s;
   
@@ -185,7 +188,8 @@ srp_process_init_msg(struct dh_instance *self, struct lsh_string *packet)
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_KEXSRP_INIT)
       && ( (name = parse_string_copy(&buffer) ))
-      && parse_bignum(&buffer, self->e)
+      /* FIXME: Pass a more restrictive limit to parse_bignum. */
+      && parse_bignum(&buffer, self->e, 0)
       && (mpz_cmp_ui(self->e, 1) > 0)
       && GROUP_RANGE(self->method->G, self->e)
       && parse_eod(&buffer) )
@@ -279,7 +283,8 @@ srp_process_reply_msg(struct dh_instance *dh, struct lsh_string *packet)
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_KEXSRP_REPLY)
       && ( (salt = parse_string_copy(&buffer) ))
-      && parse_bignum(&buffer, dh->f)
+      /* FIXME: Pass a more restrictive limit to parse_bignum. */
+      && parse_bignum(&buffer, dh->f, 0)
       && (mpz_cmp_ui(dh->f, 1) > 0)
       && GROUP_RANGE(dh->method->G, dh->f)
       && parse_eod(&buffer))
