@@ -2,7 +2,7 @@
  *
  * Interact with the user.
  *
- * $Id: unix_interact.c,v 1.3 2001/02/20 17:49:20 nisse Exp $ */
+ * $Id: unix_interact.c,v 1.7 2001/05/01 18:59:59 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -57,6 +57,48 @@ static void winch_handler(int signum)
   window_changed = 1;
 }
 
+static struct termios original_mode;
+
+static int tty_fd = -1;
+
+static void
+stop_handler(int signum)
+{
+  struct termios mine;
+  int need_reset;
+  
+  assert(signum == SIGTSTP);
+
+  need_reset = tty_getattr(tty_fd, &mine);
+  tty_setattr(tty_fd, &original_mode);
+
+  kill(getpid(), SIGSTOP);
+
+  if (need_reset)
+    tty_setattr(tty_fd, &mine);
+}
+
+static void
+install_suspend_handler(int fd)
+{
+  struct sigaction stop;
+
+  memset(&stop, 0, sizeof(stop));
+  stop.sa_handler = stop_handler;
+  sigemptyset(&stop.sa_mask);
+  stop.sa_flags = 0;
+
+  tty_fd = fd;
+
+  if (!tty_getattr(fd, &original_mode))
+    werror("install_suspend_handler: tty_getattr failed (errno = %i): %z\n",
+	   errno, STRERROR(errno));
+  
+  else if (sigaction(SIGTSTP, &stop, NULL) < 0)
+    werror("Failed to install SIGTSTP handler (errno = %i): %z\n",
+	   errno, STRERROR(errno));
+}
+
 /* Depends on the tty being line buffered */
 static int
 read_line(int fd, UINT32 size, UINT8 *buffer)
@@ -158,32 +200,36 @@ unix_is_tty(struct interact *s)
 static struct lsh_string *
 unix_read_password(struct interact *s UNUSED,
 		   UINT32 max_length UNUSED,
-		   struct lsh_string *prompt, int free)
+		   const struct lsh_string *prompt, int free)
 {
-  /* NOTE: Ignores max_length; instead getpass()'s limit applies. */
-
-  char *password;
+  /* NOTE: Ignores max_length; instead getpass's limit applies. */
   
-  prompt = make_cstring(prompt, free);
-
-  if (!prompt)
-    return NULL;
+  char *password;
+  const char *cprompt = lsh_get_cstring(prompt);
 
+  if (!cprompt)
+    {
+      if (free)
+	lsh_string_free(prompt);
+      return NULL;
+    }
   /* NOTE: This function uses a static buffer. */
-  password = getpass(prompt->data);
-
-  lsh_string_free(prompt);
+  password = getpass(cprompt);
 
+  if (free)
+    lsh_string_free(prompt);
+  
   if (!password)
     return NULL;
   
-  return format_cstring(password);
+  return make_string(password);
 }
 
 
 static int
 unix_yes_or_no(struct interact *s,
-	       struct lsh_string *prompt, int def, int free)
+	       const struct lsh_string *prompt,
+	       int def, int free)
 {
 #define TTY_BUFSIZE 10
 
@@ -288,9 +334,7 @@ unix_window_size(struct interact *s,
   CAST(unix_interact, self, s);
 
   return IS_TTY(self)
-    && tty_getwinsize(self->tty_fd,
-		      &d->char_width, &d->char_height,
-		      &d->pixel_width, &d->pixel_height);
+    && tty_getwinsize(self->tty_fd, d);
 }
 
 static struct resource *
@@ -392,6 +436,8 @@ make_unix_interact(struct io_backend *backend)
 
       io_signal_handler(backend, &window_changed,
 			make_winch_handler(self));
+
+      install_suspend_handler(self->tty_fd);
     }
   return &self->super;
 }
