@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: werror.c,v 1.51 2001/03/25 12:31:10 nisse Exp $ */
+ * $Id: werror.c,v 1.57 2001/04/17 19:59:32 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -43,6 +43,10 @@
 #include <unistd.h>
 #endif
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+
 #if HAVE_SYSLOG_H
 #include <syslog.h>
 #endif
@@ -54,8 +58,11 @@ int quiet_flag = 0;
 int verbose_flag = 0;
 int syslog_flag = 0;
 
+static const char *program_name = NULL;
+
 #define WERROR_TRACE -1
 #define WERROR_DEBUG -2
+#define WERROR_LOG -3
 
 static const struct argp_option
 werror_options[] =
@@ -64,17 +71,23 @@ werror_options[] =
   { "verbose", 'v', NULL, 0, "Verbose diagnostic messages", 0},
   { "trace", WERROR_TRACE, NULL, 0, "Detailed trace", 0 },
   { "debug", WERROR_DEBUG, NULL, 0, "Print huge amounts of debug information", 0 },
+  { "log-file", WERROR_LOG, "File name", 0,
+    "Append messages to this file.", 0},
   { NULL, 0, NULL, 0, NULL, 0 }
 };
 
 static error_t
-werror_argp_parser(int key, char *arg UNUSED,
-		   struct argp_state *state UNUSED)
+werror_argp_parser(int key, char *arg,
+		   struct argp_state *state)
 {
   switch(key)
     {
     default:
       return ARGP_ERR_UNKNOWN;
+    case ARGP_KEY_END:
+    case ARGP_KEY_INIT:
+      program_name = state->name;
+      break;
     case 'q':
       quiet_flag = 1;
       break;
@@ -87,6 +100,19 @@ werror_argp_parser(int key, char *arg UNUSED,
     case WERROR_DEBUG:
       debug_flag = 1;
       break;
+    case WERROR_LOG:
+      {
+	/* FIXME: For clients, this is right: We only get lsh-related
+	 * messages to the log file, and child processes are not
+	 * affected. But for the server, perhaps we should also dup
+	 * the logfile over stderr? */
+	
+	int fd = open(arg, O_WRONLY | O_CREAT | O_APPEND, 0666);
+	if (fd < 0)
+	  argp_error(state, "Failed to open log file `%s'.", arg);
+	else
+	  set_error_stream(fd);
+      }
     }
   return 0;
 }
@@ -111,24 +137,24 @@ static const struct exception *
 static const struct exception *
 write_syslog(int fd UNUSED, UINT32 length, const UINT8 *data)
 {
-  struct lsh_string *s = make_cstring_l(length, data);
+  struct lsh_string *s;
 
-  /* NOTE: make_cstring fails if the data contains any NUL characters. */
-  assert(s);
+  /* Data must not contain any NUL:s */
+  assert(!memchr(data, '\0', length));
+  
+  /* NUL-terminate the string. */
+  s = ssh_format("%ls", length, data);
 
-#if 0
-  /* Make sure the message is properly terminated with \0. */
-  snprintf(string_buffer, MIN(BUF_SIZE, length), "%s", data);
-#endif
   /* FIXME: Should we use different log levels for werror, verbose and
    * debug? */
   
-  syslog(LOG_NOTICE, "%s", s->data);
+  syslog(LOG_NOTICE, "%s", lsh_get_cstring(s));
   lsh_string_free(s);
   
   return NULL;
 }
 
+/* FIXME: Delete argument and use program_name. */
 void
 set_error_syslog(const char *id)
 {
@@ -144,11 +170,18 @@ write_ignore(int fd UNUSED,
 { return NULL; }
 
 void
-set_error_stream(int fd, int with_poll)
+set_error_stream(int fd)
 {
   error_fd = fd;
 
-  error_write = with_poll ? write_raw_with_poll : write_raw;
+  error_write = write_raw;
+}
+
+void
+set_error_nonblocking(int fd)
+{
+  if (error_fd == fd)
+    error_write = write_raw_with_poll;
 }
 
 int
@@ -346,6 +379,12 @@ werror_paranoia_putc(UINT8 c)
 void
 werror_vformat(const char *f, va_list args)
 {
+  if (program_name)
+    {
+      werror_write(strlen(program_name), program_name);
+      werror_write(2, ": ");
+    }
+  
   while (*f)
     {
       if (*f == '%')
@@ -395,9 +434,12 @@ werror_vformat(const char *f, va_list args)
 		if (do_hex)
 		  werror_hexdump(strlen(s), s);
 
-		while (*s)
-		  (do_paranoia ? werror_paranoia_putc : werror_putc)(*s++);
-
+		else if (do_paranoia)
+		  while (*s)
+		    werror_paranoia_putc(*s++);
+		else
+		  werror_write(strlen(s), s);
+		
 		break;
 	      }
 	    case 'a':
@@ -494,7 +536,9 @@ werror(const char *format, ...)
 {
   va_list args;
 
-  if (!quiet_flag)
+  /* It is somewhat reasonable to use both -q and -v. In this case
+   * werror()-messages should be displayed. */
+  if (verbose_flag || !quiet_flag)
     {
       va_start(args, format);
       werror_vformat(format, args);
